/*****************************************************************************
 *
 *              SHT15.c -- Sensirion Temerature and Humidity Sensor
 *
 *****************************************************************************
 * FileName:        SHT15.c
 * Dependencies:
 * Processor:       PIC18
 * Compiler:        C18 02.20.00 or higher
 *
 * Linker:          MPLINK 03.40.00 or higher
 * Company:         None 
 *
 *
 *****************************************************************************
 * File Description:
 *
 *   This file provides configuration and basic hardware functionality for the 
 *   Big-Bang I2C "like protocol" of Sensirion Temperature and Humidity Sensor
 
 *
 * Change History:
 *  Rev   Date         Description
 *  0.1   12/30/2010    Initial revision (C18 only compatible)
 *****************************************************************************/

#include "SystemProfile.h"
#include "Compiler.h"
#include "HardwareProfile.h"
#include "SHT15.h"
#include <delays.h>
#include "ConfigApp.h"

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Read out the humidity and temperature data
 *
 * Note:             Usage example
 *				
 *				  	unsigned int temperature;
 *					unsigned int humidity;
 *					unsigned char checksum;
 *	
 *					sht15_read((unsigned char*) &temperature, (unsigned char*) &humidity, &checksum);
 *                  
 *                  
 ********************************************************************/

void sht15_read(unsigned char *t_value, unsigned char *h_value, unsigned char *p_checksum)
{
	unsigned int i;

	#if defined(HARVEST)
 	SDA_PU_EN = 1;
	Nop();Nop();Nop();
	#endif
    //Issue command start
    sht15_start();
	//Now send command code
    sht15_send_byte(CHECK_TEMP);				// Read Temperature
	// wait for the conversion
	#if defined(HARVEST)
	Nop();Nop();Nop();Nop();	
	SDA_PU_EN = 0;
	delay_ms(75);
	SDA_PU_EN = 1;
	delay_ms(1);
	#endif
    for(i = 0; i < 0xFFFFFF; i++) {
        if(SDA_Input == 0) {
            break;                             //wait until sensor has finished the measurement
        }
    }

    *(t_value+1) = s_read_byte(ACK);         //read the first byte (MSB)
    *(t_value) = s_read_byte(ACK);             //read the second byte (LSB)
    *p_checksum = s_read_byte(noACK);         //read checksum
	
    //Issue command start
    sht15_start();
	//Now send command code
    sht15_send_byte(CHECK_HUMD);				// Read Temperature
	// wait for the conversion
	#if defined(HARVEST)
	Nop();Nop();Nop();Nop();	
	SDA_PU_EN = 0;
	delay_ms(75);
	SDA_PU_EN = 1;
	delay_ms(1);
	#endif
    for(i = 0; i < 0xFFFFFF; i++) {
        if(SDA_Input == 0) {
            break;                             //wait until sensor has finished the measurement
        }
    }

    *(h_value+1) = s_read_byte(ACK);         //read the first byte (MSB)
    *(h_value) = s_read_byte(ACK);             //read the second byte (LSB)
    *p_checksum = s_read_byte(noACK);         //read checksum
	#if defined(HARVEST)
	Nop();Nop();Nop();	
	SDA_PU_EN = 0;
	#endif


}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Write the Status Register
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/

unsigned char s_write_statusreg(unsigned char value) {
    unsigned char error = 0;
 
	#if defined(HARVEST)
 	SDA_PU_EN = 1;
	Nop();Nop();Nop();
	#endif
    sht15_start();                         //transmission start
	sht15_send_byte(WRITE_STAT);
	sht15_send_byte(value);
	#if defined(HARVEST)
	Nop();Nop();Nop();
 	SDA_PU_EN = 0;
	#endif
 
     return error;                             //error>=1 in case of no response form the sensor
}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Read the Status Register 
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/

unsigned char s_read_statusreg(void) {

	unsigned char stat_byte_msb = 0;
	unsigned char crc;

	#if defined(HARVEST)
 	SDA_PU_EN = 1;
	Nop();Nop();Nop();	
	#endif
    sht15_start();
    sht15_send_byte(CHECK_STAT);
	stat_byte_msb  = s_read_byte(1);
	crc  = s_read_byte(0);
	#if defined(HARVEST)
	Nop();Nop();Nop();
 	SDA_PU_EN = 0;
	#endif

	return stat_byte_msb;
}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Send a Byte (Used for command or Status Register)
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/

unsigned char sht15_send_byte(unsigned char sht15_command)
{
    unsigned char i;
	unsigned char temp = sht15_command;
	unsigned char error;

    WRITE_sda();

    for(i = 0 ; i < 8 ; i++)	
    {
		_SCK = 0;
		_asm
		rlcf temp, 1,0
		_endasm
        SDA_Output = STATUSbits.C;
        _SCK = 1;
		Nop();
		Nop();
		Nop();
    }
	_SCK = 0;
    //Wait for SHT15 to acknowledge.
	READ_sda();	
	Nop();
	_SCK = 1;
	Nop();
	error = SDA_Input;
	Nop();
	_SCK = 0;
	return(error);


}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Reset the Sensor I2C interface
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/

void sht15_reset(void){
	unsigned char i;
	
	// Initial State
	#if defined(HARVEST)
	SDA_PU_EN = 1;
	Nop();Nop();Nop();
	#endif 
	_SCK = 0;
	WRITE_sda();
	SDA_Output = 1;

	for(i=0;i<8;i++){
		_SCK = 1;	
		Nop();
		_SCK = 0;
	}	
	sht15_start();
	#if defined(HARVEST)
	Nop();Nop();Nop();
	SDA_PU_EN = 0;	
	#endif
	
}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Specific SHT start command
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/

//
void sht15_start(void)
{
    WRITE_sda();

    SDA_Output = 1;
    _SCK = 0;
	Nop();
	_SCK = 1;
	Nop();
	SDA_Output = 0;
	Nop();
	_SCK = 0;
	Nop();
	Nop();
	Nop();
	_SCK =1;
	Nop();
	SDA_Output = 1;
	Nop();
	_SCK = 0; 
}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        Read 8 bits from the SHT sensor
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/


unsigned char s_read_byte(unsigned char ack) {
    unsigned char i;
    unsigned char val = 0;

    READ_sda();             //release DATA-line
 
    for(i = 0x80; i > 0; i/=2) { 
        _SCK = 1;         //clk for SENSI-BUS

        //delay_ms(1);
		Nop();Nop();Nop();Nop();

        if(SDA_Input) {
            val = (val | i);
        }
 
        _SCK = 0;
    }
    if(ack){
		WRITE_sda();
		SDA_Output = 0;
	}
	_SCK = 1;    
	Nop();Nop();Nop();            // .5uS //pulswith approx. .5 us
    _SCK = 0;  
    READ_sda();                //release DATA-line
 
    return val;
}

/*********************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    
 *
 * Overview:        General short delay
 *
 * Note:             
 *                  
 *                  
 ********************************************************************/

void delay_ms(unsigned int x)
{
	unsigned int y;
	// Tcy = 0.5us at 8Mhz
	for(y=0; y<x; y++){
		Delay1KTCYx(1);
		Delay1KTCYx(1);
	}
}